% main_v1.m     : GDP analysis: averaging
%
% AUTHOR	    : Michiel de Pooter, Rene Segers, Francesco Ravazzolo, Herman K. van Dijk                 %
%																										  %
% CREATED       : May 25, 2008 (FR)                                                                       %
% LAST UPDATE   : May 25, 2008 (FR)								    					     	          %
%                                                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
close all; clear all; clc; 
%addpath([pwd '\procs\']);

% set input and output directory
inputdir='C:\My ebooks\paper near-boundary\program\input\';%
outputdir='C:\My eBooks\paper near-boundary\program\output\prova\';%
addpath([pwd '\procs\']);

%%%%%%%%%%% control parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
MCMCndraws=2000;       % the number of required MCMC draws 
MCMCburnin=200;        % the burn-in period for the MCMC sampler
KFburn=20;            % the burn-in period for the Kalman Filter
b_draws=2;              % draws to select to reduce autocorrelation among simulations 
b_seldr=[MCMCburnin+b_draws:b_draws:MCMCndraws]'; % final draws
b_aut=1;
b_trend=1;
b_all=1;
h=60;                 % forecasts  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
randn('state',200);       % set the seed for the normal distribution
rand('state',200);        % set the seed for the uniform distribution


% IMPORTA DATA: GDP, GDPgrowth,CPI
[Y]=importdata('C:\My ebooks\paper near-boundary\program\input\quarterly_data.mat');
X1=importdata('C:\My ebooks\paper near-boundary\program\output\x_averaging_1.mat');
X2=importdata('C:\My ebooks\paper near-boundary\program\output\x_averaging_2.mat');
X4=importdata('C:\My ebooks\paper near-boundary\program\output\x_averaging_4.mat');

% Very weak informative priors around 0
Prmean=0;
Prvar=1000;

% linear averaging
%% ALL MODELS
if(b_all==1)
    all=[1,2,3,4,5];
    yfor=zeros(60,3);
    x1=[ones(110,1),X1(:,all)];
    m=size(x1,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y1=Y(103:192,2);
    for t=1:h;
        yt=y1(1:29+t,1);
        xt=x1(20:48+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        bal0(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,1)=x1(48+t+1,:)*bal0(t,:)';
    end;    
    
    x2=[ones(109,1),X2(1:109,all)];
    m=size(x2,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y2=Y(103:192,2);
    for t=1:h-1;
        yt=y2(2:29+t,1);
        xt=x2(20:47+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        bbal1(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,2)=x2(47+t+1,:)*bbal1(t,:)';
    end;
    
    x3=[ones(107,1),X4(1:107,all)];
    m=size(x3,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y3=Y(103:192,2);
    for t=1:h-3;
        yt=y3(4:29+t,1);
        xt=x3(20:45+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        bal3(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,3)=x3(45+t+1,:)*bal3(t,:)';
    end;
    BMA_all=yfor;
    savefile = [outputdir 'BMA_all.mat'];
    save(savefile,'BMA_all');
end; 
%% AUTOREGRESSIVE MODELS
if(b_aut==1)
    yfor=zeros(60,3);
    x1=[ones(110,1),X1(:,[1,2])];
    m=size(x1,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y1=Y(103:192,2);
    for t=1:h;
        yt=y1(1:29+t,1);
        xt=x1(20:48+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        baut0(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,1)=x1(48+t+1,:)*baut0(t,:)';
    end;    
    
    x2=[ones(109,1),X2(1:109,[1,2])];
    m=size(x2,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y2=Y(103:192,2);
    for t=1:h-1;
        yt=y2(2:29+t,1);
        xt=x2(20:47+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        baut1(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,2)=x2(47+t+1,:)*baut1(t,:)';
    end;
    
    x3=[ones(107,1),X4(1:107,[1,2])];
    m=size(x3,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y3=Y(103:192,2);
    for t=1:h-3;
        yt=y3(4:29+t,1);
        xt=x3(20:45+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        baut2(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,3)=x3(45+t+1,:)*baut2(t,:)';
    end;
    BMA_aut=yfor;
    savefile = [outputdir 'BMA_aut.mat'];
    save(savefile,'BMA_aut');
end; 
%% TREND MODELS
if(b_trend==1)
    yfor=zeros(60,3);
    x1=[ones(110,1),X1(:,[3,4,5])];
    m=size(x1,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y1=Y(103:192,2);
    for t=1:h;
        yt=y1(1:29+t,1);
        xt=x1(20:48+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        btr0(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,1)=x1(48+t+1,:)*btr0(t,:)';
    end;    
    
    x2=[ones(109,1),X2(1:109,[3,4,5])];
    m=size(x2,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y2=Y(103:192,2);
    for t=1:h-1;
        yt=y2(2:29+t,1);
        xt=x2(20:47+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        btr1(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,2)=x2(47+t+1,:)*btr1(t,:)';
    end;
    
    x3=[ones(107,1),X4(1:107,[3,4,5])];
    m=size(x3,2);
    bpr1=ones(m,1)*Prmean;
    bpr2=eye(m)*Prvar;
    y3=Y(103:192,2);
    for t=1:h-3;
        yt=y3(4:29+t,1);
        xt=x3(20:45+t,:);
        Vb=inv(inv(bpr2)+xt'*xt);
        btr2(t,:)=(Vb*(inv(bpr2)*bpr1+(xt'*xt)*(inv(xt'*xt)*xt'*yt)))';
        yfor(t,3)=x3(45+t+1,:)*btr2(t,:)';
    end;
    BMA_tr=yfor;
    savefile = [outputdir 'BMA_tr.mat'];
    save(savefile,'BMA_tr');
end; 