% main_v1.m     : CPI analysis
%
% AUTHOR	    : Michiel de Pooter, Rene Segers, Francesco Ravazzolo, Herman K. van Dijk                 %
%																										  %
% CREATED       : September 16, 2007 (FR)                                                                       %
% LAST UPDATE   : September 16, 2007 (FR)								    					     	          %
%                                                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
close all; clear all; clc; 
%addpath([pwd '\procs\']);

% set input and output directory
inputdir='C:\My ebooks\paper near-boundary\program\input\';%
outputdir='C:\My eBooks\paper near-boundary\program\output\';%
addpath([pwd '\procs\']);

%%%%%%%%%%% control parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
MCMCndraws=24000;       % the number of required MCMC draws 
MCMCburnin=4000;        % the burn-in period for the MCMC sampler
KFburn=20;            % the burn-in period for the Kalman Filter
b_draws=2;              % draws to select to reduce autocorrelation among simulations 
b_seldr=[MCMCburnin+b_draws:b_draws:MCMCndraws]'; % final draws
b_CPI=1;                % boolen for CPI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
randn('state',200);       % set the seed for the normal distribution
rand('state',200);        % set the seed for the uniform distribution


% IMPORTA DATA: GDP, GDPgrowth,CPI
Y=importdata('C:\My ebooks\paper near-boundary\program\input\quarterly_data.mat');

if(b_CPI==1)
    
    % Construct dependent variable
    y=Y(99:189,3);
    [T,N]=size(y);
    h=43;          % number of forecasts
    To=146;          % initial in-sample period in forecasting exersies
    rhomax=0.9999;
    % full in-sample analysis
    x=ones(T,1);                % constant
    x1=x;
    y1=Y(98:188,3);               % lags of Y
    M=size(x,2);
    MCMC_b=zeros(MCMCndraws,M);
    MCMC_R=zeros(MCMCndraws,N);
    MCMC_rho=zeros(MCMCndraws,N);
    MCMC_R(1,1)=var(y);
    MCMC_b(1,1)=mean(y);
    for i=1:MCMCndraws-1;
        %rho
        xrho=y1-x1*MCMC_b(i,:);
        yrho=y-x*MCMC_b(i,:);
        rhom=inv(xrho'*xrho)*xrho'*yrho;
        rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
        MCMC_rho(i+1,1)=min(rho,rhomax);
        %beta
        xx=x-x1*MCMC_rho(i+1,:);
        yx=y-y1*MCMC_rho(i+1,:);
        bm=inv(xx'*xx)*xx'*yx;
        MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
        %sigma
        sig1=normrnd(0,1,T,N);
        sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
        MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
    end;
    bstatistics=[mean(MCMC_b(b_seldr,:)),std(MCMC_b(b_seldr,:))];
    Rstatistics=[mean(MCMC_R(b_seldr,:)),std(MCMC_R(b_seldr,:))];
    rhostatistics=[mean(MCMC_rho(b_seldr,:)),std(MCMC_rho(b_seldr,:))];
    CPIgrowth_parameters_m1 = [MCMC_b,MCMC_rho,MCMC_R];
    
    % forecasts
    mMCMC_y=zeros(size(b_seldr,1),h);   % density forecasts
    mpMCMC_y=zeros(h,1);                % point forecasts
    for t=1:h;
        y=Y(99:To+t-1,3);
        [T,N]=size(y);
        x=ones(T,1);
        x1=x;
        y1=Y(98:To+t-2,3);
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_y=zeros(MCMCndraws,N);
        MCMC_R(1,1)=s2OLS;
        MCMC_b(1,1)=mean(Y(3:192,2));
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rho(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            MCMC_y(i+1,1)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,1)+(y(T,1)-MCMC_b(i+1,1))*MCMC_rho(i+1,1);
        end;
        mpMCMC_y(t,1)=mean(MCMC_y(b_seldr,1));
        mMCMC_y(:,t)=MCMC_y(b_seldr,:);
    end
    CPIgrowth_forecasts_m1 = [mpMCMC_y';mMCMC_y];
end

clc;
disp('Program done!');