% main_v1.m     : GDP analysis
%
% AUTHOR	    : Michiel de Pooter, Rene Segers, Francesco Ravazzolo, Herman K. van Dijk                 %
%																										  %
% CREATED       : September 16, 2007 (FR)                                                                       %
% LAST UPDATE   : May 20, 2008       (FR)								    					     	          %
%                                                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
close all; clear all; clc; 
%addpath([pwd '\procs\']);

% set input and output directory
inputdir='C:\My ebooks\paper near-boundary\program\input\';%
outputdir='C:\My eBooks\paper near-boundary\program\output\';%
addpath([pwd '\procs\']);

%%
%%%%%%%%%%% control parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
MCMCndraws=24000;       % the number of required MCMC draws 
MCMCburnin=4000;        % the burn-in period for the MCMC sampler
KFburn=20;            % the burn-in period for the Kalman Filter
b_draws=2;              % draws to select to reduce autocorrelation among simulations 
b_seldr=[MCMCburnin+b_draws:b_draws:MCMCndraws]'; % final draws

b_constgrowth=1;        % boolen for constant growth rate
b_secm=1;               % booolen for simple ECM
b_ecm=1;                % booolen for ECM
b_trend=1;              % boolen for trend
b_timevar=1;            % boolen for time-varying 

b_insample=1;           % booolen for in-sample analysis

hh=12;                  % maximum forecast horizons
%%
randn('state',200);       % set the seed for the normal distribution
rand('state',200);        % set the seed for the uniform distribution


% IMPORTA DATA: GDP, GDPgrowth,CPI
[Y]=importdata('C:\My ebooks\paper near-boundary\program\input\quarterly_data.mat');

%% RANDOM WALK + DRIFT
if(b_constgrowth==1)
    % Construct dependent variable
    h=110;          % number of forecasts
    To=82;          % initial in-sample period in forecasting exersies

    % full in-sample analysis
    if(b_insample==1)
        y=Y(3:192,2);    
        [T,N]=size(y);
        x=ones(T,1);
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_R(1,1)=s2OLS;
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(x'*x))^0.5)*normrnd(0,1)+bOLS;
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_b(i+1,:))'*(y-x*MCMC_b(i+1,:));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        bstatistics=[mean(MCMC_b(b_seldr,:)),std(MCMC_b(b_seldr,:))];
        Rstatistics=[mean(MCMC_R(b_seldr,:)),std(MCMC_R(b_seldr,:))];
        GDPgrowth_parameters_m1 = [MCMC_b,MCMC_R];
    end

    % forecasts
    mpMCMC_y=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,2);
        [T,N]=size(y);
        x=ones(T,1);
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_y=zeros(MCMCndraws,N);
        MCMC_R(1,1)=s2OLS;
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(x'*x))^0.5)*normrnd(0,1)+bOLS;
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_b(i+1,:))'*(y-x*MCMC_b(i+1,:));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            for f=1:hh;
                MCMC_y(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,:);
            end
        end
        mpMCMC_y(t,:)=mean(MCMC_y(b_seldr,:));
    end
    GDPgrowth_forecasts_m1 = mpMCMC_y;
end
%% SIMPLE ECM +TREND

if(b_secm==1)
    h=110;          % number of forecasts
    To=82;          % initial in-sample period in forecasting exersies
    rhomax=0.99;
    
    if(b_insample==1)
        y=Y(3:192,2);
        [T,N]=size(y);
        % full in-sample analysis
        x=ones(T,1);                % constant
        x1=x;
        y1=Y(2:191,2);               % lags of Y
        M=size(x,2);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_rho=zeros(MCMCndraws,N);
        MCMC_R(1,1)=var(y);
        MCMC_b(1,1)=mean(y);
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rho(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        bstatistics=[mean(MCMC_b(b_seldr,:)),std(MCMC_b(b_seldr,:))];
        Rstatistics=[mean(MCMC_R(b_seldr,:)),std(MCMC_R(b_seldr,:))];
        rhostatistics=[mean(MCMC_rho(b_seldr,:)),std(MCMC_rho(b_seldr,:))];
        GDPgrowth_parameters_m2 = [MCMC_b,MCMC_rho,MCMC_R];
    end

    % forecasts
    mpMCMC_y=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,2);
        [T,N]=size(y);
        x=ones(T,1);
        x1=x;
        y1=Y(2:To+t-2,2);
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_y=zeros(MCMCndraws,N);
        MCMC_R(1,1)=s2OLS;
        MCMC_b(1,1)=mean(Y(3:192,2));
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rho(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            xf=y(T,1);
            for f=1:hh
                MCMC_y(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,1)+(xf-MCMC_b(i+1,1))*MCMC_rho(i+1,1);
                xf=MCMC_y(i+1,f);
            end
        end
        mpMCMC_y(t,:)=mean(MCMC_y(b_seldr,:));
    end
    GDPgrowth_forecasts_m2 = [mpMCMC_y];
end
%%
if(b_ecm==1)
        % Construct dependent variable
        h=110;          % number of forecasts
        To=82;          % initial in-sample period in forecasting exersies
        y=Y(3:192,2);
        [T,N]=size(y);
        trend=zeros(T,1); % construct trend
        for tr=1:T;
            trend(tr,1)=tr;
        end;    
        y=Y(3:192,2);
            [T,N]=size(y);
        if(b_insample==1)
            
            y=Y(3:192,2);
            [T,N]=size(y);
            x=trend;                % constant
            x1=[trend-1];
            y1=Y(2:191,1);               % lags of Y
            y2=Y(2:191,2);
            M=size(x,2);
            MCMC_b=zeros(MCMCndraws,2);
            MCMC_R=zeros(MCMCndraws,N);
            MCMC_rho=zeros(MCMCndraws,2);
            MCMC_R(1,1)=var(y);
            MCMC_b(1,:)=[0.0004,0.8];
            for i=1:MCMCndraws-1;
                %rho
                xrho=[y1-MCMC_b(i,1)-x1*MCMC_b(i,2),-(y2-MCMC_b(i,2))];
                yrho=y-MCMC_b(i,2);
                rhom=inv(xrho'*xrho)*xrho'*yrho;
                rho=chol(MCMC_R(i,1)*inv(xrho'*xrho))'*normrnd(0,1,2,1)+rhom;
                rho(1,1)=min(rho(1,1),-0.05);
                MCMC_rho(i+1,1)=rho(1,1)-rho(2,1)+1;
                MCMC_rho(i+1,2)=rho(2,1);
                %beta
                xx=[-ones(T,1)*rho(1,1),ones(T,1)-x1*rho(1,1)+MCMC_rho(i+1,2)];
                %xx=[-ones(T,1)*rho(1,1),-rho(1,1)*(x1-MCMC_rho(i+1,2))];
                yx=y-y1*rho(1,1)+y2*MCMC_rho(i+1,2);
                bm=inv(xx'*xx)*xx'*yx;
                MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(xx'*xx))'*normrnd(0,1,2,1)+bm)';
                %sigma
                sig1=normrnd(0,1,T,N);
                sig2=(y-MCMC_b(i+1,2)-rho(1,1)*(y1-MCMC_b(i+1,1)-x1*MCMC_b(i+1,2))+rho(2,1)*(y2-MCMC_b(i+1,2)))'*(y-MCMC_b(i+1,2)-rho(1,1)*(y1-MCMC_b(i+1,1)-x1*MCMC_b(i+1,2))+rho(2,1)*(y2-MCMC_b(i+1,2)));
                MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            end;
            bstatistics=[mean(MCMC_b(b_seldr,:));std(MCMC_b(b_seldr,:))];
            Rstatistics=[mean(MCMC_R(b_seldr,:));std(MCMC_R(b_seldr,:))];
            rhostatistics=[mean(MCMC_rho(b_seldr,:));std(MCMC_rho(b_seldr,:))];
            GDPgrowth_parameters_m3 = [MCMC_b,MCMC_rho,MCMC_R];
        end

        % forecasts
        mpMCMC_y=zeros(h,hh);
        for t=1:h;
            y=Y(3:To+t-1,2);
            [T,N]=size(y);
            x=trend(1:T,1);                % constant 
            x1=[trend(1:T)-1];
            y1=Y(2:To+t-1-1,1);               % lags of Y
            y2=Y(1:To+t-1-2,2);
            MCMC_b=zeros(MCMCndraws,2);
            MCMC_R=zeros(MCMCndraws,N);
            MCMC_rho=zeros(MCMCndraws,2);
            MCMC_y=zeros(MCMCndraws,N);
            MCMC_R(1,1)=var(Y(3:192,2));
            MCMC_b(1,:)=[0.8,0.0001];
            for i=1:MCMCndraws-1;
                %rho
                xrho=[y1-MCMC_b(i,1)-x1*MCMC_b(i,2),-(y2-MCMC_b(i,2))];
                yrho=y-MCMC_b(i,2);
                rhom=inv(xrho'*xrho)*xrho'*yrho;
                rho=chol(MCMC_R(i,1)*inv(xrho'*xrho))'*normrnd(0,1,2,1)+rhom; 
                rho(1,1)=min(rho(1,1),-0.05);
                MCMC_rho(i+1,1)=rho(1,1)-rho(2,1)+1;
                MCMC_rho(i+1,2)=rho(2,1);
                %beta
                xx=[-ones(T,1)*rho(1,1),ones(T,1)-x1*rho(1,1)+MCMC_rho(i+1,2)];
                yx=y-y1*rho(1,1)+y2*MCMC_rho(i+1,2);
                bm=inv(xx'*xx)*xx'*yx;
                MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(xx'*xx))'*normrnd(0,1,2,1)+bm)'; 
                %sigma
                sig1=normrnd(0,1,T,N);
                sig2=(y-MCMC_b(i+1,2)-rho(1,1)*(y1-MCMC_b(i+1,1)-x1*MCMC_b(i+1,2))+rho(2,1)*(y2-MCMC_b(i+1,2)))'*(y-MCMC_b(i+1,2)-rho(1,1)*(y1-MCMC_b(i+1,1)-x1*MCMC_b(i+1,2))+rho(2,1)*(y2-MCMC_b(i+1,2)));
                MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
                xf1=Y(T,1);
                xf2=Y(T-1,2);
                for f=1:hh;
                    MCMC_y(i+1,f)=(MCMC_R(i+1,:).^0.5)*normrnd(0,1)+MCMC_b(i+1,2)+rho(1,1)*(Y(T,1)-MCMC_b(i+1,1)-(T+f-1)*MCMC_b(i+1,2))-rho(2,1)*(xf2-MCMC_b(i+1,2));
                    xf1=MCMC_y(i+1,f)-xf1;
                    xf2=MCMC_y(i+1,f);
                end
            end;
            mpMCMC_y(t,:)=mean(MCMC_y(b_seldr,:));
        end
        GDPgrowth_forecasts_m3 = mpMCMC_y;
end
%% AUTOREGRESSIVE + ADDITTIVE TREND

if(b_trend==1)
    % Construct dependent variable
    h=110;            % number of forecasts
    To=82;            % initial in-sample period in forecasting exersies
    
    y=Y(3:192,1);
    [T,N]=size(y);
    trend=zeros(T,1); % construct trend
    for tr=1:T;
        trend(tr,1)=tr;
    end;
    y1=Y(2:191,1);
    y2=Y(1:190,1);

    if(b_insample==1)
        x=[ones(T,1),trend,y1,y2];
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        bOLSG=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_R(1,1)=s2OLS;
        check=zeros(MCMCndraws,1);
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(x'*x))'*normrnd(0,1,M,1)+bOLS)';
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_b(i+1,:)')'*(y-x*MCMC_b(i+1,:)');
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        bstatistics=[mean(MCMC_b(b_seldr,:));std(MCMC_b(b_seldr,:))];
        Rstatistics=[mean(MCMC_R(b_seldr,:));std(MCMC_R(b_seldr,:))];
        GDPgrwoth_parameters_m4= [MCMC_b,MCMC_R];
    end
    
    % forecasts
    mpMCMC_y=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,1);
        [T,N]=size(y);
        y1=Y(2:To+t-2,1);
        y2=Y(1:To+t-3,1);
        x=[ones(T,1),trend(1:T,:),y1,y2];
        M=size(x,2);
        bOLS=inv(x'*x)*x'*y;
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_yl=zeros(MCMCndraws,hh);
        MCMC_y=zeros(MCMCndraws,hh);
        MCMC_R(1,1)=s2OLS;
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(x'*x))'*normrnd(0,1,M,1)+bOLS)';
            sig2=(y-x*MCMC_b(i+1,:)')'*(y-x*MCMC_b(i+1,:)');
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            xf=[1,T+1,y(T,1),y(T-1,1)];
            yprev=y(T,1);
            for f=1:hh
                MCMC_yl(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+xf*MCMC_b(i+1,:)';
                if(f>1)
                    xf=[1,T+f+1,MCMC_yl(i+1,f),MCMC_yl(i+1,f-1)];
                else
                    xf=[1,T+f+1,MCMC_yl(i+1,f),y(T,1)];
                end
                MCMC_y(i+1,f)=(MCMC_yl(i+1,f)-yprev)*100;
                yprev=MCMC_yl(i+1,f);
            end
        end;
        mpMCMC_y(t,:)=mean(MCMC_y(b_seldr,:));
    end
    GDPgrowth_forecasts_m4 = mpMCMC_y;
end

%% TIME VARYING-RW
if(b_timevar==1)
    % full in-sample
    h=110;            % number of forecasts
    To=82;            % initial in-sample period in forecasting exersies
    
    
    if(b_insample==1)
        y=Y(3:192,2);
        [T,N]=size(y);
        MCMC_b=zeros(MCMCndraws,T);
        MCMC_R=zeros(MCMCndraws,1);
        MCMC_Q=zeros(MCMCndraws,1);
        MCMC_R(1,1)=0.1;
        MCMC_b(1,:)=ones(1,T)*mean(y);
        MCMC_Q(1,1)=0.0001;
        b0i=mean(y);
        for i=1:MCMCndraws
            [bt,Pt,bt_T,Pt_T,ft_tlag]=kalman_filter_aff(y,ones(T,1,1),[],[],ones(T,1)*MCMC_R(i,1),0,1,[],MCMC_Q(i,:),b0i,0.001);
            bt_T=[b0i'; bt_T];
            temp=zeros(T+1,1);
            temp(1,:)=0.0001;
            temp(2:T+1,:)=Pt_T;
            Pt_T=temp;
            [b_post]=kalman_sampler_aff(bt_T,Pt_T,1,[],[],MCMC_Q(i,:),0,1); % with SMOOTHED Kalman estimates
            % store this draw for beta
            MCMC_b(i+1,:)=b_post;
            % discard KF values
            b_post0=b_post(1,KFburn+1:T)';
            b_post1=b_post(1,KFburn:T-1)';
            % draw R
            sig1=normrnd(0,1,T-KFburn,1);
            sig2=(y(KFburn+1:T,1)-b_post0)'*(y(KFburn+1:T,1)-b_post0);
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            %draw Q
            Q1=normrnd(0,1,T-KFburn,1);
            Q2=(b_post0-b_post1)'*(b_post0-b_post1);
            MCMC_Q(i+1,:)=Q2*inv(Q1'*Q1);
        end
        btimevar_m1=[mean(MCMC_b(b_seldr,:));std(MCMC_b(b_seldr,:));quantile(MCMC_b(b_seldr,:),0.75);quantile(MCMC_b(b_seldr,:),0.25)];
        Rstatistics=[mean(MCMC_R(b_seldr,:));std(MCMC_R(b_seldr,:))];
        Qstatistics=[mean(MCMC_Q(b_seldr,:));std(MCMC_Q(b_seldr,:))];
        GDPtimevar_parameters = [MCMC_R,MCMC_Q];
    end
    
    % forecasts
    mpMCMC_y=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,2);
        [T,N]=size(y);
        MCMC_b=zeros(MCMCndraws,T);
        MCMC_c=zeros(MCMCndraws,2);
        MCMC_R=zeros(MCMCndraws,1);
        MCMC_Q=zeros(MCMCndraws,1);
        MCMC_R(1,1)=0.1;
        MCMC_b(1,:)=ones(1,T)*mean(y);
        MCMC_Q(1,1)=0.0001;
        MCMC_(1,:)=[0,1];
        b0i=mean(y);
        for i=1:MCMCndraws
            [bt,Pt,bt_T,Pt_T,ft_tlag]=kalman_filter_aff(y,ones(T,1,1),[],[],ones(T,1)*MCMC_R(i,1),0,1,[],MCMC_Q(i,:),b0i,0.001);
            bt_T=[b0i'; bt_T];
            temp=zeros(T+1,1);
            temp(1,:)=0.001;
            temp(2:T+1,:)=Pt_T;
            Pt_T=temp;
            [b_post]=kalman_sampler_aff(bt_T,Pt_T,1,[],[],MCMC_Q(i,:),0,1); % with SMOOTHED Kalman estimates
            % store this draw for beta
            MCMC_b(i+1,:)=b_post;
            % discard KF values
            b_post0=b_post(1,KFburn+1:T)';
            b_post1=b_post(1,KFburn:T-1)';
            % draw R
            sig1=normrnd(0,1,T-KFburn,1);
            sig2=(y(KFburn+1:T,1)-b_post0)'*(y(KFburn+1:T,1)-b_post0);
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            %draw Q
            Q1=normrnd(0,1,T-KFburn,1);
            Q2=(b_post0-b_post1)'*(b_post0-b_post1);
            MCMC_Q(i+1,:)=Q2*inv(Q1'*Q1);
            bfor=MCMC_b(i+1,T);
            for f=1:hh
                MCMC_y(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+bfor; 
                bfor=(MCMC_Q(i+1,:)^0.5)*normrnd(0,1)+bfor;
            end
        end
        mpMCMC_y(t,:)=mean(MCMC_y(b_seldr,:));
    end
    GDPtimevar_forecasts = [mpMCMC_y];
end


clc;
disp('Program done!');