% main_v1.m     : DY and TB analysis
%
% AUTHOR	    : Michiel de Pooter, Rene Segers, Francesco Ravazzolo, Herman K. van Dijk                 %
%																										  %
% CREATED       : September 16, 2007 (FR)                                                                       %
% LAST UPDATE   : May 20, 2008       (FR)								    					     	          %
%                                                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
close all; clear all; clc; 
%addpath([pwd '\procs\']);

% set input and output directory
inputdir='C:\My ebooks\paper near-boundary\program\input\';%
outputdir='C:\My eBooks\paper near-boundary\program\output\';%
addpath([pwd '\procs\']);

%%%%%%%%%%% control parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
MCMCndraws=24000;       % the number of required MCMC draws 
MCMCburnin=4000;        % the burn-in period for the MCMC sampler
KFburn=20;            % the burn-in period for the Kalman Filter
b_draws=2;              % draws to select to reduce autocorrelation among simulations 
b_seldr=[MCMCburnin+b_draws:b_draws:MCMCndraws]'; % final draws
b_TB=1;                 % boolen for TB
b_DY=1;                 % booolen for DY
b_insample=1;           % booolen for in-sample analysis
hh=12;                  % maximum forecast horizons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
randn('state',200);       % set the seed for the normal distribution
rand('state',200);        % set the seed for the uniform distribution


% IMPORTA DATA: GDP, GDPgrowth,CPI
Y=importdata('C:\My ebooks\paper near-boundary\program\input\monthly_data.mat');

if(b_TB==1)
    
    h=331;       % number of forecasts
    To=242;       % initial in-sample period in forecasting exersies
    
    if(b_insample==1)
        % full in-sample analysis
        % UNRESTRICTED
        % Construct dependent variable
        y=Y(3:573,2);
        [T,N]=size(y);
        y1=Y(2:572,2);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_rho=zeros(MCMCndraws,N);
        MCMC_R(1,1)=var(y);
        MCMC_b(1,1)=mean(y);
        
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        
        % RESTRICTED by acception
        % Construct dependent variable
        y=Y(3:573,2);
        [T,N]=size(y);
        rhomax=0.99;
        y1=Y(2:572,2);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_brr=zeros(MCMCndraws,M);
        MCMC_Rrr=zeros(MCMCndraws,N);
        MCMC_rhorr=zeros(MCMCndraws,N);
        MCMC_Rrr(1,1)=var(y);
        MCMC_brr(1,1)=mean(y);
        
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_brr(i,:);
            yrho=y-x*MCMC_brr(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_Rrr(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rhorr(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rhorr(i+1,:);
            yx=y-y1*MCMC_rhorr(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_brr(i+1,:)=((MCMC_Rrr(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_brr(i+1,:))-(y1-x1*MCMC_brr(i+1))*MCMC_rhorr(i+1,1))'*((y-x*MCMC_brr(i+1,:))-(y1-x1*MCMC_brr(i+1))*MCMC_rhorr(i+1,1));
            MCMC_Rrr(i+1,:)=sig2*inv(sig1'*sig1);
        end;


        % RESTRICTED to root
        y=Y(3:573,2)-Y(2:572,2);
        MCMC_br=zeros(MCMCndraws,1);
        MCMC_Rr=zeros(MCMCndraws,N);
        MCMC_Rr(1,1)=std(y)^2;
        bOLS=mean(y);
        x=ones(T,1);
        for i=1:MCMCndraws-1;
            MCMC_br(i+1,:)=(chol(MCMC_Rr(i,1)*inv(x'*x))'*normrnd(0,1)+bOLS)';
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_br(i+1,:))'*(y-x*MCMC_br(i+1,:)');
            MCMC_Rr(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        bstatistics=[mean(MCMC_b(b_seldr,:)),mean(MCMC_brr(b_seldr,:)),mean(MCMC_br(b_seldr,:));std(MCMC_b(b_seldr,:)),std(MCMC_brr(b_seldr,:)),std(MCMC_br(b_seldr,:))];
        Rstatistics=[mean(MCMC_R(b_seldr,:)),mean(MCMC_Rrr(b_seldr,:)),mean(MCMC_Rr(b_seldr,:));std(MCMC_R(b_seldr,:)),std(MCMC_Rrr(b_seldr,:)),std(MCMC_Rr(b_seldr,:))];
        rhostatistics=[mean(MCMC_rho(b_seldr,:)),mean(MCMC_rhorr(b_seldr,:)),1;std(MCMC_rho(b_seldr,:)),std(MCMC_rhorr(b_seldr,:)),0];
    end

    % forecasts
    % RESTRICTED by accception
    mpMCMC_yrr=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,2);
        [T,N]=size(y);
        y1=Y(2:To+t-2,2);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_rho=zeros(MCMCndraws,N);
        MCMC_yrr=zeros(MCMCndraws,N*hh);
        MCMC_b(1,1)=mean(y);
        MCMC_R(1,1)=var(y);
        rhomax=0.99;         % unrestricted
        for i=1:MCMCndraws-1
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rho(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            xf=y(T,1);
            for f=1:hh
                MCMC_yrr(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,1)+(xf-MCMC_b(i+1,1))*MCMC_rho(i+1,1);
                xf=MCMC_yrr(i+1,f);
            end
        end
        mpMCMC_yrr(t,:)=mean(MCMC_yrr(b_seldr,:));
    end
    
    % RESTRICTED by root
    mpMCMC_yr=zeros(h,hh);
    for t=1:h;
        y=Y(3:To+t-1,2)-Y(2:To+t-2,2);
        yfor=Y(To+t-1,2);
        [T,N]=size(y);
        x=ones(T,1);
        M=size(x,2);
        bOLS=mean(y);
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,1);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_yr=zeros(MCMCndraws,N*hh);
        MCMC_R(1,1)=s2OLS;
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(x'*x))'*normrnd(0,1,M,1)+bOLS)';
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_b(i+1,:)')'*(y-x*MCMC_b(i+1,:)');
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            yin=yfor;
            for f=1:hh
                MCMC_yr(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,:)'+yin;
                yin=MCMC_yr(i+1,f);
            end
        end
        mpMCMC_yr(t,:)=mean(MCMC_yr(b_seldr,:));
    end
    
    TB_forecasts = [mpMCMC_yrr,mpMCMC_yr];
end

if(b_DY==1)
    
    h=312;           % number of forecasts
    To=242;          % initial in-sample period in forecasting exersies
        
    if(b_insample==1)
        % full in-sample analysis
        % UNRESTRICTED
        y=Y(75:554,3);
        [T,N]=size(y);
        y1=Y(74:553,3);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_rho=zeros(MCMCndraws,N);
        MCMC_R(1,1)=var(y);
        MCMC_b(1,1)=mean(y);
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        
        %  RESTRICTED by acception
        % Construct dependent variable
        y=Y(75:554,3);
        [T,N]=size(y);
        rhomax=0.99;
        y1=Y(74:553,3);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_brr=zeros(MCMCndraws,M);
        MCMC_Rrr=zeros(MCMCndraws,N);
        MCMC_rhorr=zeros(MCMCndraws,N);
        MCMC_Rrr(1,1)=var(y);
        MCMC_brr(1,1)=mean(y);
        for i=1:MCMCndraws-1;
            %rho
            xrho=y1-x1*MCMC_brr(i,:);
            yrho=y-x*MCMC_brr(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_Rrr(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rhorr(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rhorr(i+1,:);
            yx=y-y1*MCMC_rhorr(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_brr(i+1,:)=((MCMC_Rrr(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_brr(i+1,:))-(y1-x1*MCMC_brr(i+1))*MCMC_rhorr(i+1,1))'*((y-x*MCMC_brr(i+1,:))-(y1-x1*MCMC_brr(i+1))*MCMC_rhorr(i+1,1));
            MCMC_Rrr(i+1,:)=sig2*inv(sig1'*sig1);
        end;

        % RESTRICTED to root
        y=Y(75:554,3)-Y(74:553,3);
        MCMC_br=zeros(MCMCndraws,1);
        MCMC_Rr=zeros(MCMCndraws,N);
        MCMC_Rr(1,1)=std(y)^2;
        bOLS=mean(y);
        x=ones(T,1);
        for i=1:MCMCndraws-1;
            MCMC_br(i+1,:)=(chol(MCMC_Rr(i,1)*inv(x'*x))'*normrnd(0,1)+bOLS)';
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_br(i+1,:))'*(y-x*MCMC_br(i+1,:)');
            MCMC_Rr(i+1,:)=sig2*inv(sig1'*sig1);
        end;
        
        bstatistics=[mean(MCMC_b(b_seldr,:)),mean(MCMC_brr(b_seldr,:)),mean(MCMC_br(b_seldr,:));std(MCMC_b(b_seldr,:)),std(MCMC_brr(b_seldr,:)),std(MCMC_br(b_seldr,:))];
        Rstatistics=[mean(MCMC_R(b_seldr,:)),mean(MCMC_Rrr(b_seldr,:)),mean(MCMC_Rr(b_seldr,:));std(MCMC_R(b_seldr,:)),std(MCMC_Rrr(b_seldr,:)),std(MCMC_Rr(b_seldr,:))];
        rhostatistics=[mean(MCMC_rho(b_seldr,:)),mean(MCMC_rhorr(b_seldr,:)),1;std(MCMC_rho(b_seldr,:)),std(MCMC_rhorr(b_seldr,:)),0];
        DY_parameters = [MCMC_b,MCMC_rho,MCMC_R,MCMC_brr,MCMC_rhorr,MCMC_Rrr,MCMC_br,MCMC_Rr];
    end

    % forecasts
    % RESTRICTED by accception
    mpMCMC_yrr=zeros(h,hh);
    for t=1:h;
        y=Y(75:To+t-1,3);
        [T,N]=size(y);
        y1=Y(74:To+t-2,3);
        x=ones(T,1);
        x1=x;
        M=size(x,2);
        MCMC_b=zeros(MCMCndraws,M);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_rho=zeros(MCMCndraws,N);
        MCMC_yrr=zeros(MCMCndraws,N*hh);
        MCMC_b(1,1)=mean(y);
        MCMC_R(1,1)=var(y);
        rhomax=0.99;         % unrestricted
        for i=1:MCMCndraws-1
            %rho
            xrho=y1-x1*MCMC_b(i,:);
            yrho=y-x*MCMC_b(i,:);
            rhom=inv(xrho'*xrho)*xrho'*yrho;
            rho=((MCMC_R(i,1)*inv(xrho'*xrho))^0.5)*normrnd(0,1)+rhom;
            MCMC_rho(i+1,1)=min(rho,rhomax);
            %beta
            xx=x-x1*MCMC_rho(i+1,:);
            yx=y-y1*MCMC_rho(i+1,:);
            bm=inv(xx'*xx)*xx'*yx;
            MCMC_b(i+1,:)=((MCMC_R(i,1)*inv(xx'*xx))^0.5)*normrnd(0,1)+bm;
            %sigma
            sig1=normrnd(0,1,T,N);
            sig2=((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1))'*((y-x*MCMC_b(i+1,:))-(y1-x1*MCMC_b(i+1))*MCMC_rho(i+1,1));
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            xf=y(T,1);
            for f=1:hh
                MCMC_yrr(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,1)+(xf-MCMC_b(i+1,1))*MCMC_rho(i+1,1);
                xf=MCMC_yrr(i+1,f);
            end
        end
        mpMCMC_yrr(t,:)=mean(MCMC_yrr(b_seldr,:));
    end
    
    % RESTRICTED by root
    mpMCMC_yr=zeros(h,hh);
    for t=1:h;
        y=Y(75:To+t-1,3)-Y(74:To+t-2,3);
        yfor=Y(To+t-1,3);
        [T,N]=size(y);
        x=ones(T,1);
        M=size(x,2);
        bOLS=mean(y);
        s2OLS=(y-x*bOLS)'*(y-x*bOLS)/(T-M);
        MCMC_b=zeros(MCMCndraws,1);
        MCMC_R=zeros(MCMCndraws,N);
        MCMC_yr=zeros(MCMCndraws,N*hh);
        MCMC_R(1,1)=s2OLS;
        for i=1:MCMCndraws-1;
            MCMC_b(i+1,:)=(chol(MCMC_R(i,1)*inv(x'*x))'*normrnd(0,1,M,1)+bOLS)';
            sig1=normrnd(0,1,T,N);
            sig2=(y-x*MCMC_b(i+1,:)')'*(y-x*MCMC_b(i+1,:)');
            MCMC_R(i+1,:)=sig2*inv(sig1'*sig1);
            yin=yfor;
            for f=1:hh
                MCMC_yr(i+1,f)=(MCMC_R(i+1,:)^0.5)*normrnd(0,1)+MCMC_b(i+1,:)'+yin;
                yin=MCMC_yr(i+1,f);
            end
        end
        mpMCMC_yr(t,:)=mean(MCMC_yr(b_seldr,:));
    end
    
    DY_forecasts = [mpMCMC_yrr,mpMCMC_yr];
end

clc;
disp('Program done!');