% FIRST YOU NEED TO RUN THE kalman_filter_aff.m function to get bt_t and Pt_t
 
% Simulating the latent factor:

% bT|yT ~ N(bT|T , PT|T)       for t = T 

% bt|yt,b(t+1) ~ N(bt|(b(t+1) , Pt|(b(t+1))

% where
% bt|b(t+1) = bt|t + Pt|t*Ft'*(F*Pt|t*F'+Q)^(-1)*(b(t+1)-mt-Ft*bt|t-Gt*Xt)
% and
% Pt|b(t+1) = Pt|t - Pt|t*Ft'*(Ft*Pt|t*Ft' + Q)^(-1)*Ft*Pt|t

function [b_post] =  kalman_sampler_aff(bt, Pt, Ft, Xt, Gt, Q, mt, ndraws);

% b_post: ndraws x nobs x nstates 

% nobs is no of observations + the one-step ahead foecasts
nobs = size(bt,1);
nstates = size(bt,2);

Pt_t = zeros(nobs-1, nstates, nstates);
bt_t = bt(2:nobs, :);
Pt_t = Pt(2:nobs, :, :);

% get bT|T and PT|T from the last iteration of the kalman filter
bT_T = bt(nobs,:);
PT_T = zeros(nstates, nstates);
PT_T(:,:) = Pt(nobs,:,:);
% ***********************************************************
nobs = nobs-1;

% initialize array that will store the numerical posterior distribution:
% nobs x nstates       is one path of this distribtuion.....
% .... and we draw       ndraws    of these paths
bt_path = zeros(ndraws, nobs, nstates);

for i = 1:ndraws
    % draw bt for t = T 
    bt_path(i, nobs, :) = (chol(PT_T)'*(mvnrnd(zeros(nstates,1), eye(nstates)))')'+bT_T;
    
    for j = 0:nobs-2
        % derive first the mean bt_t+1 and variance Pt_t+1 for the draws of bt for
        % t = T-1, ... , 1
        part1 = reshape(Pt_t(nobs-1-j, :, :), nstates, nstates) *Ft';
        part2 = inv(Ft*reshape(Pt_t(nobs-1-j, :, :), nstates, nstates)*Ft'+Q);
        if size(Xt,1)>0
            part3 = reshape(bt_path(i, nobs-j,:),1,nstates)'- mt - Ft*bt_t(nobs-1-j,:)'-Gt*Xt(nobs-1-j,:)';       
        else
            part3 = reshape(bt_path(i, nobs-j,:),1,nstates)'- mt - Ft*bt_t(nobs-1-j,:)';   
        end
        bt_tpl1 = bt_t(nobs-1-j,:)' + part1*part2*part3;

        part4 = reshape(Pt_t(nobs-1-j,:,:), nstates, nstates)*Ft';
        part5 = part2;
        part6 = Ft*reshape(Pt_t(nobs-1-j,:,:), nstates, nstates);
        Pt_tpl1 = reshape(Pt_t(nobs-1-j,:,:), nstates, nstates) - part4*part5*part6;
        
        % now draw bt for t = T-1, ... , 1
        [R1tr,pchol1] = cholupdate(Pt_tpl1,eye(nstates,1),'-');
        if(pchol1<1)
            bt_path(i, nobs-1-j, :) = (chol(Pt_tpl1)'*(mvnrnd(zeros(nstates,1), eye(nstates)))')'+bt_tpl1';
        else
            bt_path(i, nobs-1-j, :) = bt_tpl1';
        end
                
    end
end

b_post = bt_path;